#include <Wire.h> //include library for I2C
#include <LiquidCrystal_I2C.h> //include library for LCD
#include<Servo.h> //include library for servo motor
#include "RTClib.h" //include library for RTC

Servo myservo;

int incomingByte = 0;
LiquidCrystal_I2C lcd(0x27, 20, 4);
RTC_DS1307 rtc;

#define BUTTON_DIR DDRB
#define BUTTON_IN PINB
#define welcome_menu 1
#define timer_status 2
#define CLOCK_ADDRESS 0xD0
#define buzzer 12

//time in rtc
unsigned char rtcHour = 00;
unsigned char rtcMinute = 00;
unsigned char rtcSecond = 00;

//time offset
unsigned char oHour = 00;
unsigned char oMinute = 00;
unsigned char oSecond = 20;

//mean time of the system
unsigned char mHour = 00;
unsigned char mMinute = 00;
unsigned char mSecond = 00;

//time to feed
unsigned char fHour = 00;
unsigned char fMinute = 00;
unsigned char fSecond = 00;

volatile unsigned int menu_num = 1;
volatile unsigned int prev_menu_num = menu_num;
volatile bool feeding_mode, manual_mode = false, schedule_mode = true;
volatile bool feedStatus;
volatile unsigned int feeding_proportion = 1;
volatile int timeFocus;
//angka menu_num menandakan tempat menu saat ini berada:
//1 adalah menu depan
//2 adalah status timer dan buzzer
//3 adalah display sisa makanan di wadah
//4 adalah menu settings
//5 adalah menu pilihan setting timer
//6 adalah menu pilihan setting takaran
//7 adalah menu pilihan setting kekerasan buzzer (mati, ... %, Max)
//9 adalah menu mengatur setting timer
//10 adalah menu mengatur setting takaran


void setup() {
  Serial.begin(115200);
  Wire.begin();

  BUTTON_DIR = (1 << DDB0) | (1 << DDB1) | (1 << DDB2) | (1 << DDB3);
  lcd.init();
  lcd.backlight();
  PCICR = (1 << PCIE0);
  PCMSK0 = ((1 << PCINT0) | (1 << PCINT1) | (1 << PCINT2) | (1 << PCINT3)); //PCINT0 Ok, PCINT1 Back/Decline, PCINT2 right/up, PCINT 3 left/down
  pinMode(buzzer, OUTPUT);
  feeding_mode = manual_mode;
  rtcCheck();
  myservo.attach(3);
  myservo.write(30);
}

// declaring service routine for buttons in port B
ISR(PCINT0_vect) {
  if (BUTTON_IN & (1 << PB0)) { //ok button
    ok();
  } else if (BUTTON_IN & (1 << PB1)) { //back button
    back();
  } else if (BUTTON_IN & (1 << PB2)) { //the right/ up / increment button
    right_up();
  } else if (BUTTON_IN & (1 << PB3)) { //the left/ down/ decrement button
    left_down();
  }
}

void loop() {
  rtcUpdate(); //untuk update waktu saat ini
  menuOperate(); //untuk menampilkan menu

}

void rtcUpdate() {
  DateTime now = rtc.now();
  if (menu_num == welcome_menu) {
    if (rtcSecond != now.second()) {
      menu();
    }
  }
  rtcHour = (now.hour());
  rtcMinute = (now.minute());
  rtcSecond = (now.second());
}

void rtcCheck() {
  while (!Serial); // for Leonardo/Micro/Zero
  if (! rtc.begin()) {
    Serial.println("Couldn't find RTC");
    while (1);
  }
  if (! rtc.isrunning()) {
    Serial.println("RTC is NOT running!");
    // following line sets the RTC to the date & time this sketch was compiled
    rtc.adjust(DateTime(F(__DATE__), F(__TIME__)));
    // This line sets the RTC with an explicit date & time, for example to set
    // January 21, 2014 at 3am you would call:
    // rtc.adjust(DateTime(2014, 1, 21, 3, 0, 0));
  }
}
void menuOperate() {
  if (digitalRead(10) || digitalRead(11) || digitalRead(9) || digitalRead(8)) {
    menu();
  }
  if (feeding_mode == schedule_mode) {
    if (rtcHour == fHour) {
      if (rtcMinute == fMinute) {
        if (rtcSecond == fSecond) {
          feedStatus = true;
          digitalWrite(buzzer, HIGH);
        } else digitalWrite(buzzer, LOW);
      }
    }
  } else if (feeding_mode == manual_mode) {
    if (Serial.available() > 0) {
      // read the incoming byte:
      incomingByte = Serial.read();
      Serial.println(incomingByte);
      if (incomingByte == 45) {
        feedStatus = 1;
      }
    }
  }
  if (feedStatus == 1) {
    feedServo();
    feedStatus = 0;
  }
}

void feedServo() {
  volatile int i = 0;
  do {
    myservo.write(120); //jalur makanan terbuka
    _delay_ms(500); //menentukan seberapa banyak makanan yang keluar
    myservo.write(30); //jalur makanan tertutup
    i++;
    _delay_ms(250);
  } while (i < feeding_proportion);
  digitalWrite(buzzer, HIGH);
  _delay_ms(400);
  digitalWrite(buzzer, LOW);

  //  for(i = 0; i < feeding_proportion; i++){
  //    myservo.write(120); //jalur makanan terbuka
  //    delay(500); //menentukan seberapa banyak makanan yang keluar
  //    myservo.write(30); //jalur makanan tertutup
  //  }

}

void menu() {
  lcd.clear();
  if (menu_num == welcome_menu) { //Menu 1
    //display welcome menu greet
    lcd.print("Hello User!");

    //display the current hour
    if (rtcHour >= 0 && rtcHour <= 9) {
      lcd.setCursor(0, 1);
      lcd.print("0");
      lcd.setCursor(1, 1);
      lcd.print(rtcHour);
    } else {
      lcd.setCursor(0, 1);
      lcd.print(rtcHour);//the current mean time
    }
    lcd.setCursor(2, 1);
    lcd.print(":");

    //display the current minute
    if (rtcMinute >= 0 && rtcMinute <= 9) {
      lcd.setCursor(3, 1);
      lcd.print("0");
      lcd.setCursor(4, 1);
      lcd.print(rtcMinute);
    } else {
      lcd.setCursor(3, 1);
      lcd.print(rtcMinute);
    }
    lcd.setCursor(5, 1);
    lcd.print(":");

    //display the current second
    if (rtcSecond >= 0 && rtcSecond <= 9) {
      lcd.setCursor(6, 1);
      lcd.print("0");
      lcd.setCursor(7, 1);
      lcd.print(rtcSecond);
    } else {
      lcd.setCursor(6, 1);
      lcd.print(rtcSecond);
    }
  } else if (menu_num == timer_status) { //Menu 2
    //print current timer set
    lcd.print("Timer:");
    if (feeding_mode == manual_mode) {
      lcd.setCursor(6, 0);
      lcd.print("Manual");
    } else if (feeding_mode == schedule_mode) {
      if (fHour >= 0 && fHour <= 9) {
        lcd.setCursor(6, 0);
        lcd.print("0");
        lcd.setCursor(7, 0);
        lcd.print(fHour);
      } else {
        lcd.setCursor(6, 0);
        lcd.print(fHour);//the current mean time
      }
      lcd.setCursor(8, 0);
      lcd.print(":");

      if (fMinute >= 0 && fMinute <= 9) {
        lcd.setCursor(9, 0);
        lcd.print("0");
        lcd.setCursor(10, 0);
        lcd.print(fMinute);
      } else {
        lcd.setCursor(9, 0);
        lcd.print(fMinute);
      }
      lcd.setCursor(11, 0);
      lcd.print(":");
      if (fSecond >= 0 && fSecond <= 9) {
        lcd.setCursor(12, 0);
        lcd.print("0");
        lcd.setCursor(13, 0);
        lcd.print(fSecond);
      } else {
        lcd.setCursor(12, 0);
        lcd.print(fSecond);
      }
    }
  } else if (menu_num == 3) {
    //print current food portion
    lcd.print("Food Portion:");
    lcd.setCursor(0, 1 );
    lcd.print(feeding_proportion);
  } else if (menu_num == 4) {
    lcd.print("Settings");
    lcd.setCursor(0, 1);
    lcd.print("Press OK");
  } else if (menu_num > 4 && menu_num < 8) {
    settingsMenu();
  } else if (menu_num == 8) {
    lcd.print("Current Mode:");
    lcd.setCursor(0, 1);
    if (feeding_mode == manual_mode) {
      lcd.print("Manual Feed");
    } else if (feeding_mode == schedule_mode) {
      lcd.print("Scheduled Feed");
    }
  } else if (menu_num == 9) {
    lcd.print("Feeding Time");
    if (fHour >= 0 && fHour <= 9) {
      lcd.setCursor(0, 1);
      lcd.print("0");
      lcd.setCursor(1, 1);
      lcd.print(fHour);
    } else {
      lcd.setCursor(0, 1);
      lcd.print(fHour);//the current mean time
    }
    lcd.setCursor(2, 1);
    lcd.print(":");

    if (fMinute >= 0 && fMinute <= 9) {
      lcd.setCursor(3, 1);
      lcd.print("0");
      lcd.setCursor(4, 1);
      lcd.print(fMinute);
    } else {
      lcd.setCursor(3, 1);
      lcd.print(fMinute);
    }
    lcd.setCursor(5, 1);
    lcd.print(":");
    if (fSecond >= 0 && fSecond <= 9) {
      lcd.setCursor(6, 1);
      lcd.print("0");
      lcd.setCursor(7, 1);
      lcd.print(fSecond);
    } else {
      lcd.setCursor(6, 1);
      lcd.print(fSecond);
    }
  } else if (menu_num == 10) {
    lcd.print("Food Portion:");
    lcd.setCursor(0, 1);
    lcd.print(feeding_proportion);
  }
}

void settingsMenu() {
  if (menu_num == 5) {
    lcd.print("Mode Settings");
    lcd.setCursor(0, 1);
    lcd.print("OK to Continue");
  }
  else if (menu_num == 6) {
    lcd.print("Timer Settings");
    lcd.setCursor(0, 1);
    lcd.print("OK to Continue");
  }
  else if (menu_num == 7) {
    lcd.print("Feeding Settings");
    lcd.setCursor(0, 1);
    lcd.print("OK to Continue");
  }
}

//MENU NUMBER LIMITS
void mainMenu_limit() { //this is the function to loop the menu when button is pressed over the ammount of menu
  if (menu_num == 0) {
    menu_num = 4;
  } else if (menu_num == 5) {
    menu_num = 1;
  }
}
void settingMenu_limit() {
  if (menu_num < 5) {
    menu_num = 7;
  } else if (menu_num > 7) {
    menu_num = 5;
  } else if (menu_num == 6) {
    if (feeding_mode == 0) {
      if (digitalRead(10)) {
        menu_num = 7;
      } else if (digitalRead(11)) {
        menu_num = 5;
      }
    }
  }
}
//BUTTON FUNCTIONS
void left_down() { //function when button down/left is pressed
  //Main menu decrement
  if (menu_num >= 1 && menu_num <= 4) {
    menu_num--;
    mainMenu_limit();
  } else if (menu_num >= 5 && menu_num <= 7) {
    menu_num--;
    settingMenu_limit();
  } else if (menu_num == 8) {
    mode_shift();
  } else if (menu_num == 9) {
    //function mengatur waktu memberi makanan
    if (timeFocus == 0) {
      fHour--;
      Serial.print(fHour);
    } else if (timeFocus == 1) {
      if (fMinute >= 0 && fMinute <= 59) {
        fMinute--;
      }
      Serial.print(fMinute);
    } else if (timeFocus == 2) {
      if (fSecond >= 0 && fSecond <= 59) {
        fSecond--;
      }
      Serial.print(fSecond);
    }
    timerSettingLimit();
  } else if (menu_num == 10) {
    feeding_proportion --;
    feeding_proportion_limit();
  }
}

//function when button right/up/increment is pressed
void right_up() {
  if (menu_num >= 1 && menu_num <= 4) {
    menu_num++;
    mainMenu_limit();
  } else if (menu_num >= 5 && menu_num <= 7) {
    menu_num++;
    settingMenu_limit();
  } else if (menu_num == 8) {
    mode_shift();
  } else if (menu_num == 9) {
    if (timeFocus == 0) {
      if (fHour >= 0 && fHour <= 23) {
        fHour++;
      }
    } else if (timeFocus == 1) {
      if (fMinute >= 0 && fMinute <= 59) {
        fMinute++;
      }
    } else if (timeFocus == 2) {
      if (fSecond >= 0 && fSecond <= 59) {
        fSecond++;
      }
    }
    timerSettingLimit();
  } else if (menu_num == 10) {
    feeding_proportion ++;
    feeding_proportion_limit();
  }
}

//batesin angka di menu timer settings
void timerSettingLimit() {
  if (fHour == 255) {
    fHour = 23;
  } else if (fHour > 23) {
    fHour = 0;
  }
  if (fMinute == 255) {
    fMinute = 59;
  } else if (fMinute > 59) {
    fMinute = 0;
  }
  if (fSecond == 255) {
    fSecond = 59;
  } else if (fSecond > 59) {
    fSecond = 0;
  }
}

void mode_shift() {
  if (feeding_mode == manual_mode) {
    feeding_mode = schedule_mode;
  } else if (feeding_mode == schedule_mode) {
    feeding_mode = manual_mode;
  }
}
void feeding_proportion_limit() {
  if (feeding_proportion > 10) {
    feeding_proportion = 10;
  } else if (feeding_proportion < 1) {
    feeding_proportion = 1;
  }
}
void ok() {
  if (menu_num == 4) {
    menu_num++;
  } else if (menu_num == 5) {
    menu_num = 8;
  } else if (menu_num == 6) {
    menu_num = 9;
  } else if (menu_num == 7) {
    menu_num = 10;
  } else if (menu_num == 9) {
    changeTimeFocus();
  }
}
void changeTimeFocus() {
  //0 hour
  //1 minute
  //2 second
  if (timeFocus == 0) {
    timeFocus ++;
  } else if (timeFocus == 1) {
    timeFocus ++;
  } else if (timeFocus == 2) {
    timeFocus = 0;
  }
}
void back() {
  if (menu_num >= 1 && menu_num <= 4) {
    menu_num = 1;
  } else if (menu_num > 4 && menu_num <= 7) {
    menu_num = 4;
  } else if (menu_num == 8) {
    menu_num = 5;
  } else if (menu_num == 9) {
    menu_num = 6;
    timeFocus = 0;
  } else if (menu_num == 10) {
    menu_num = 7;
  }
}
